<?php

declare(strict_types=1);

namespace Drupal\jsonapi_request_logger\EventSubscriber;

use Drupal\Core\Config\ConfigFactoryInterface;
use Psr\Log\LoggerInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpKernel\Event\RequestEvent;
use Symfony\Component\HttpKernel\KernelEvents;
use Symfony\Contracts\Translation\TranslatorTrait;

/**
 * Class that records detailed information of each JSON:API HTTP request.
 */
final class RequestLoggerSubscriber implements EventSubscriberInterface {

  use TranslatorTrait;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  private ConfigFactoryInterface $configFactory;

  /**
   * The logger service.
   *
   * @var \Psr\Log\LoggerInterface
   */
  private LoggerInterface $logger;

  /**
   * Constructor.
   */
  public function __construct(ConfigFactoryInterface $configFactory, LoggerInterface $logger) {
    $this->configFactory = $configFactory;
    $this->logger = $logger;
  }

  /**
   * Kernel request event handler.
   */
  public function onKernelRequest(RequestEvent $event): void {
    $config = $this->configFactory->get('jsonapi_request_logger.settings');

    if ($config->get('enable') && $event->getRequest()->getRequestFormat() === 'api_json') {
      $request = $event->getRequest();

      $context['@request_headers'] = json_encode($request->headers->all(), JSON_PRETTY_PRINT);
      $context['@request_query'] = $request->getQueryString();
      $context['@request_content'] = $request->getContent();

      $message = <<<REQUEST
      <dl>
        <dt>Headers</dt>
        <dd><pre>{$context['@request_headers']}</pre></dd>
        <dt>Query</dt>
        <dd><pre>{$context['@request_query']}</pre></dd>
        <dt>Content</dt>
        <dd><pre>{$context['@request_content']}</pre></dd>
      </dl>
      REQUEST;

      $this->logger->info(t('JSON:API request received:') . $message);
    }

  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    return [
      KernelEvents::REQUEST => ['onKernelRequest'],
    ];
  }

}
